#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include "machine.h"
#include "blocks.h"
#include "pore.h"
#include "props.h"
#include "rand_step.h"
#include "gaussdev.h"
#include "ca_histo.h"
extern CA_FLOAT global_pressure;
extern int getxyz (int cellnum, int *nc, int *Cell);

/* also find the avg. frac. solid */
	int cell_find_nmols (BB_struct * bp, int sbnum, PORE_str * c_p, CA_FLOAT * fs, CA_FLOAT * sol, CA_FLOAT * alloy, int Cell[3],
			     int cellnum)
	{
	  CA_FLOAT km;
	  int errors = 0;

	  CA_FLOAT cell_vol, cell_temp, cell_tempK, cell_fs, cell_fl, cell_sol, cell_phi, cell_si;
	  CA_FLOAT cell_satSI, p_pres, p_sat;
	  CA_FLOAT extra_h, extra_nmols;
	#ifdef CHIRAZI_MULTI
	  CA_FLOAT **conc_multi;
	#endif


  /* km is short for 1-partcoef */
  km = bp->mprops.gasprops.km;


  #ifdef CHIRAZI_MULTI
    conc_multi = sp->c_sol_alloy_multi;   /*multi componenet concentration array */
  #endif

  /* figure out the temperature */
  cell_temp = bp->sb[sbnum]->c_temp[cellnum];

  /* collect the attributes of the cell */
  cell_vol = bp->vol_c;
  cell_tempK = cell_temp + STD_TEMP;    /* in Kelvin */
  cell_fs = (*(fs + cellnum));  /* the fraction solid of the cell */
  cell_fl = 1 - (*(fs + cellnum));      /* ditto fraction liqid */
  cell_sol = (*(sol + cellnum));        /* and the sol (hydrogen) content (cl) */

  /*lever rule for how much H in mushy cell */
  cell_phi = 1 - (km * cell_fs);

  /* figure out the alloy solute concentration (si or mg or cu etc)   */
  /* procedurize? */
  cell_si = 0;
  if (bp->ctrl->scheil == TRUE) {       /*schiel mode -- calculate alloy */
    cell_si = find_sch_conc (cell_tempK, cell_fl);
  } else {                      /*not scheil mode -- get alloy from array */
    if ((bp->ctrl->diffuse_alloy == TRUE) && (bp->ctrl->diffuse_alloy_multi == FALSE)) {
      cell_si = (*(alloy + cellnum));
    }
  }                             /* end scheil/notschiel */

  /* call the routine from props.c to find the saturation limit */
  if ((bp->ctrl->diffuse_alloy == TRUE) && (bp->ctrl->diffuse_alloy_multi == FALSE)) {
    cell_satSI = find_sat (&(bp->mprops), cell_tempK, cell_si, cell_fs);
  }
#ifdef CHIRAZI_MULTI
  if (bp->ctrl->diffuse_alloy_multi == TRUE) {
    cell_satSI = find_sat_multi (bp, cell_tempK, conc_multi, cell_fs, cellnum);
  }
#endif

  if (bp->ctrl->diffuse_alloy_poly == TRUE) {
    cell_satSI = find_sat_poly (bp, sbnum, cell_tempK);
  }


  /* calculate the excess hydrogen */
  p_pres = c_p->Pressure;
  p_sat = SQRT (p_pres / (global_pressure * P_AP_PA)) * cell_satSI;
  extra_h = cell_phi * (cell_sol - p_sat);
  extra_nmols = extra_h * (cell_vol);   /* mols of h atoms */
  c_p->extra_nmols += extra_nmols;
  c_p->sat = p_sat;

  /* and hence calculate the supersaturation ratio */
  c_p->supersat = (cell_sol / cell_satSI);

  /* add in the fraction solid to check if cell is frozen */
  c_p->FracSol += cell_fs;

  return (errors);
}

int gather_hydrogen (p_c_list * list)
{
/* function stub */
  return (100);
}

/* Little subroutine to get rcs id into the object code */
/* so you can use ident on the compiled program  */
/* also you can call this to print out or include the rcs id in a file */
char const *rcs_id_find_nmols_c ()
{
  static char const rcsid[] = "$Id: find_nmols.c 887 2006-03-01 18:21:01Z rcatwood $";

  return (rcsid);
}

/* end of rcs_id_subroutine */
